//	WSCParser.cpp
//
//

#include "WSCParser.hpp"
#include "StringUtility.hpp"

#include <cstring>
const std::string WSCObject::secdeliminator = "o---o";


Location::Location(const std::string& value): Location(){
	auto values = sections(value);
	int count = 0;
	for (auto &entry : values) {
		auto num = std::stoi(entry) ;
		switch (count) {
			case 0:
				x=num;
				break;
			case 1:
				y=num;
				break;
			case 2:
				z=num;
				break;
			case 3:
				map=num;
				break;

			default:
				break;
		}
		count++ ;
	}
}


WSCParser::WSCParser() {
	currentline = 0 ;
	insection = false ;
}

std::vector<WSCObject> WSCParser::objects() const {
	return entries ;
}

void WSCParser::parse(const std::string &path){
	std::ifstream filestream(path,std::ifstream::in);
	if (!filestream.is_open()) {
		return ;
	}
	filename = path ;
	this->parseStream(filestream);
	filestream.close();
}

void WSCParser::parseStream(std::istream &input){
	char buffer[4096];
	currentline = 0 ;
	insection = false ;
	currentobject = WSCObject();
	
	while (!input.eof()){
		std::memset(buffer, 0, sizeof(buffer));
		input.getline(buffer, sizeof(buffer));
		
		if (input.gcount()>0){
			std::string line(buffer) ;
			processLine(line);
		}
		currentline++;
	}
}

void WSCParser::endSection(){
	if (insection) {
		insection = false;
		currentobject.endline = currentline;
		currentobject.filename = filename;
		if (currentobject.freeze == true ){
			entries.push_back(currentobject);
		}
		currentobject = WSCObject();
	}
}

void WSCParser::processLine(const std::string &line){
	auto value = trim(line) ;
	if (value == WSCObject::secdeliminator) {
		this->endSection();
	}
	else if (line[0] == '[') {
		auto pos = line.find("]");
		auto name = line.substr(1,pos-1);
		this->startSection(str_toupper(name));
	}
	else {
		auto values = separate(line, "=");
		auto key = str_toupper(std::get<0>(values)) ;
		auto value = std::get<1>(values) ;
		this->keyvalue(key, value);
	}
}

void WSCParser::startSection(const std::string &name){
	if (name == "ITEM") {
		currentobject.type = WSCObject::Type::item ;
		insection = true ;
		currentobject.startline = currentline;
	}
	else if (name == "HOUSE") {
		currentobject.type = WSCObject::Type::house;
		insection = true ;
		currentobject.startline = currentline;
	}
	else if (name == "BOAT") {
		currentobject.type = WSCObject::Type::boat;
		insection = true ;
		currentobject.startline = currentline;
	}
}

void WSCParser::keyvalue(const std::string &key, const std::string &value) {
	if (!insection){
		return ;
	}
	if (key == "TYPE") {
		if (value != "255") {
			insection = false ;
			currentobject = WSCObject() ;
		}
		else {
			currentobject.freeze = true ;
		}
	}
	else if (key == "ID") {
		currentobject.id = std::stoi(value,0,0);
	}
	else if (key == "COLOUR"){
		currentobject.color = std::stoi(value,0,0);
	}
	else if (key == "LOCATION") {
		currentobject.loc = Location(value) ;
	}
	
}
