//	WSCParser.hpp
//
//

#ifndef WSCParser_hpp
#define WSCParser_hpp

#include <string>
#include <vector>
#include <iostream>
#include <fstream>
#include <sstream>
#include "StringUtility.hpp"

struct Location {
	int x;
	int y;
	int z;
	int map;
	Location() {
		x=-1;
		y=-1;
		z=-1;
		map = -1;
	}
	Location(const std::string& value) ;
	std::string description() const  {
		std::stringstream buffer ;
		buffer << x <<","<<y<<","<<z<<","<<map ;
		return buffer.str() ;
	}
};

struct WSCObject {
	const static std::string secdeliminator ;

	enum class Type { notype,item,boat,house};
	Type type ;
	int id ;
	Location loc ;
	int color ;
	std::string filename ;
	int startline ;
	int endline ;
	bool freeze ;
	WSCObject() {
		endline = 0 ;
		startline = 0;
		color = -1;
		id = -1 ;
		type = Type::notype;
		freeze = false ;
	}
	std::string description() const {
		std::stringstream buffer ;
		switch (type) {
			case Type::item:
				buffer <<"Item: " ;
				break;
			case Type::house:
				buffer <<"House: " ;
				break;
			case Type::boat:
				buffer <<"Boat: " ;
				break;

			default:
				break;
		}
		buffer << "0x"<<str_number(id,16) ;
		buffer << " Color: 0x"<<str_number(color,16) ;
		buffer << " Location: "<< loc.description();
		return buffer.str() ;
	}
};

class WSCParser {
    
	std::vector<WSCObject> entries ;
	bool insection ;
	WSCObject currentobject ;
	int currentline ;
	std::string filename ;
	
public:
	WSCParser();
	void parse(const std::string& path);
	void parseStream(std::istream& input);
	void startSection(const std::string& name);
	void endSection();
	void processLine(const std::string& line);
	void keyvalue(const std::string& key, const std::string&  value);
	
	std::vector<WSCObject> objects() const ;
};

#endif /* WSCParser_hpp */
